/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  GMime
 *  Copyright (C) 2000-2010 Jeffrey Stedfast
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public License
 *  as published by the Free Software Foundation; either version 2.1
 *  of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
 *  02110-1301, USA.
 */


#ifndef __GMIME_FILTER_CHARSET_H__
#define __GMIME_FILTER_CHARSET_H__

#include <iconv.h>
#include <kerio/tiny/types.h>
#include <gmime/gmime-filter.h>

G_BEGIN_DECLS

#define GMIME_TYPE_FILTER_CHARSET            (g_mime_filter_charset_get_type ())
#define GMIME_FILTER_CHARSET(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GMIME_TYPE_FILTER_CHARSET, GMimeFilterCharset))
#define GMIME_FILTER_CHARSET_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), GMIME_TYPE_FILTER_CHARSET, GMimeFilterCharsetClass))
#define GMIME_IS_FILTER_CHARSET(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GMIME_TYPE_FILTER_CHARSET))
#define GMIME_IS_FILTER_CHARSET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GMIME_TYPE_FILTER_CHARSET))
#define GMIME_FILTER_CHARSET_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GMIME_TYPE_FILTER_CHARSET, GMimeFilterCharsetClass))
typedef struct _GMimeFilterCharset GMimeFilterCharset;
typedef struct _GMimeFilterCharsetClass GMimeFilterCharsetClass;

typedef uint16 UTF16;
typedef uint32 UTF32;
typedef uint8 UTF8;

#define UNI_SUR_HIGH_START  (UTF32)0xD800
#define UNI_SUR_HIGH_END    (UTF32)0xDBFF
#define UNI_SUR_LOW_START   (UTF32)0xDC00
#define UNI_SUR_LOW_END     (UTF32)0xDFFF
#define halfShift           ((UTF32)10)
#define halfBase            ((UTF32)0x0010000UL)
#define halfMask            ((UTF32)0x3FFUL)
#define UNI_REPLACEMENT_CHAR (UTF32)0x0000FFFD


/**
 * GMimeFilterCharset:
 * @parent_object: parent #GMimeFilter
 * @from_charset: charset that the filter is converting from
 * @to_charset: charset the filter is converting to
 * @cd: charset conversion state
 *
 * A filter to convert between charsets.
 **/
struct _GMimeFilterCharset {
	GMimeFilter parent_object;
	
	char *from_charset;
	char *to_charset;
	iconv_t cd;
	gboolean invalidMutlibyte;
};

struct _GMimeFilterCharsetClass {
	GMimeFilterClass parent_class;
	
};

static const UTF8 first_byte_mark[7] = { 0x00, 0x00, 0xC0, 0xE0, 0xF0, 0xF8, 0xFC };

typedef enum {
	conversion_ok, 		/* conversion successful */
	source_exhausted,	/* partial character in source, but hit end */
	target_exhausted,	/* no room in target for conversion */
	source_illegal		/* source sequence is illegal/malformed */
} conversion_result;

GType g_mime_filter_charset_get_type (void);

GMimeFilter *g_mime_filter_charset_new (const char *from_charset, const char *to_charset);
gboolean g_mime_filter_charset_smooth_conversion(GMimeFilterCharset *filter);

G_END_DECLS

#endif /* __GMIME_FILTER_CHARSET_H__ */
