/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  GMime
 *  Copyright (C) 2000-2009 Jeffrey Stedfast
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public License
 *  as published by the Free Software Foundation; either version 2.1
 *  of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, 51 Franklin Street, Fifth Floor, Boston, MA
 *  02110-1301, USA.
 */


#ifndef __GMIME_PART_CHARSET_H__
#define __GMIME_PART_CHARSET_H__

#include <glib.h>
#include <stdio.h>

#include <gmime/gmime-charset.h>
#include <gmime/gmime-part.h>

G_BEGIN_DECLS

#define GMIME_TYPE_PART_CHARSET             (g_mime_part_charset_get_type ())
#define GMIME_PART_CHARSET(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), GMIME_TYPE_PART_CHARSET, GMimePartCharset))
#define GMIME_PART_CHARSET_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), GMIME_TYPE_PART_CHARSET, GMimePartCharsetClass))
#define GMIME_IS_PART_CHARSET(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GMIME_TYPE_PART_CHARSET))
#define GMIME_IS_PART_CHARSET_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE ((klass), GMIME_TYPE_PART_CHARSET))
#define GMIME_PART_CHARSET_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS ((obj), GMIME_TYPE_PART_CHARSET, GMimePartCharsetClass))

typedef struct _GMimePartCharset GMimePartCharset;
typedef struct _GMimePartCharsetClass GMimePartCharsetClass;

/**
 * GMimePartCharset:
 * @parent_object: parent #GMimePart
 * @charset: a charset of the part
 *
 * A leaf-node MIME part object.
 **/
struct _GMimePartCharset {
	GMimePart parent_object;

    char *charset;
    char *content_charset;
};

struct _GMimePartCharsetClass {
	GMimePartClass parent_class;

    void (* set_content_object_with_charset) (GMimePartCharset *mime_part, GMimeDataWrapper *content, const char *content_charset);
};


GType g_mime_part_charset_get_type (void);

/* constructors */
GMimePartCharset *g_mime_part_charset_new (void);
GMimePartCharset *g_mime_part_charset_new_with_type (const char *type, const char *subtype);

/* accessor functions */

const char *g_mime_part_charset_get_content_charset (GMimePartCharset *mime_part);

void g_mime_part_charset_set_content_object_with_charset(GMimePartCharset *mime_part, GMimeDataWrapper *content, const char *content_charset);

const char *g_mime_part_charset_get_best_charset(GMimePartCharset *mime_part);

G_END_DECLS

#endif /* __GMIME_PART_H__ */
